package ru.orangesoftware.financisto.widget;

import android.app.Activity;
import android.content.Intent;
import android.view.View;
import android.widget.LinearLayout;
import ru.orangesoftware.financisto.R;
import ru.orangesoftware.financisto.activity.AbstractActivity;
import ru.orangesoftware.financisto.activity.ActivityLayout;
import ru.orangesoftware.financisto.model.Account;

import static ru.orangesoftware.financisto.activity.AbstractActivity.setVisibility;

/**
 * Created by IntelliJ IDEA.
 * User: Denis Solonenko
 * Date: 6/24/11 6:45 PM
 */
public class RateLayoutView implements RateNodeOwner {

    private final AbstractActivity activity;
    private final ActivityLayout x;
    private final LinearLayout layout;

    private AmountInput amountInputFrom;
    private AmountInput amountInputTo;

    private RateNode rateNode;
    private View amountInputToNode;

    private AmountInput.OnAmountChangedListener amountFromChangeListener;
    private AmountInput.OnAmountChangedListener amountToChangeListener;
    private Account selectedAccountFrom;
    private Account selectedAccountTo;

    public RateLayoutView(AbstractActivity activity, ActivityLayout x, LinearLayout layout) {
        this.activity = activity;
        this.x = x;
        this.layout = layout;
    }

    public void setAmountFromChangeListener(AmountInput.OnAmountChangedListener amountFromChangeListener) {
        this.amountFromChangeListener = amountFromChangeListener;
    }

    public void setAmountToChangeListener(AmountInput.OnAmountChangedListener amountToChangeListener) {
        this.amountToChangeListener = amountToChangeListener;
    }

    public void createUI() {
        //amount from
        amountInputFrom = new AmountInput(activity);
        amountInputFrom.setOwner(activity);
        amountInputFrom.setExpense();
        amountInputFrom.disableIncomeExpenseButton();
        x.addEditNode(layout, R.string.amount_from, amountInputFrom);
        //amount to & rate
        amountInputTo = new AmountInput(activity);
        amountInputTo.setOwner(activity);
        amountInputTo.setIncome();
        amountInputTo.disableIncomeExpenseButton();
        amountInputToNode = x.addEditNode(layout, R.string.amount_to, amountInputTo);
        amountInputTo.setOnAmountChangedListener(onAmountToChangedListener);
        amountInputFrom.setOnAmountChangedListener(onAmountFromChangedListener);
        setVisibility(amountInputToNode, View.GONE);
        rateNode = new RateNode(this, x, layout);
        setVisibility(rateNode.rateInfoNode, View.GONE);
    }

    public void selectFromAccount(Account account) {
        selectedAccountFrom = account;
        amountInputFrom.setCurrency(account.currency);
        checkNeedRate();
    }

    public void selectToAccount(Account account) {
        selectedAccountTo = account;
        amountInputTo.setCurrency(account.currency);
        checkNeedRate();
    }

    private void checkNeedRate() {
        if (isDifferentCurrencies()) {
            setVisibility(rateNode.rateInfoNode, View.VISIBLE);
            setVisibility(amountInputToNode, View.VISIBLE);
            calculateRate();
        } else {
            setVisibility(rateNode.rateInfoNode, View.GONE);
            setVisibility(amountInputToNode, View.GONE);
        }
    }

    private void calculateRate() {
        long amountFrom = amountInputFrom.getAmount();
        long amountTo = amountInputTo.getAmount();
        float r = 1.0f*amountTo/amountFrom;
        if (!Float.isNaN(r)) {
            rateNode.setRate(r);
        }
        rateNode.updateRateInfo();
    }

    public long getFromAmount() {
        return amountInputFrom.getAmount();
    }

    public long getToAmount() {
        if (isDifferentCurrencies()) {
            return amountInputTo.getAmount();
        } else {
            return -amountInputFrom.getAmount();
        }
    }

    private boolean isDifferentCurrencies() {
        return selectedAccountFrom != null && selectedAccountTo != null &&
               selectedAccountFrom.currency.id != selectedAccountTo.currency.id;
    }

    public void onActivityResult(int requestCode, Intent data) {
        if (amountInputFrom.processActivityResult(requestCode, data)) {
            calculateRate();
            return;
        }
        if (amountInputTo.processActivityResult(requestCode, data)) {
            calculateRate();
            return;
        }
        if (requestCode == RateNode.EDIT_RATE) {
            String amount = data.getStringExtra(AmountInput.EXTRA_AMOUNT);
            if (amount != null) {
                rateNode.setRate(Float.parseFloat(amount));
                updateToAmountFromRate();
            }
        }
    }

    private final AmountInput.OnAmountChangedListener onAmountFromChangedListener = new AmountInput.OnAmountChangedListener(){
        @Override
        public void onAmountChanged(long oldAmount, long newAmount) {
            double r = rateNode.getRate();
            if (r > 0) {
                long amountFrom = amountInputFrom.getAmount();
                long amountTo = Math.round(r*amountFrom);
                amountInputTo.setOnAmountChangedListener(null);
                amountInputTo.setAmount(amountTo);
                amountInputTo.setOnAmountChangedListener(onAmountToChangedListener);
            } else {
                long amountFrom = amountInputFrom.getAmount();
                long amountTo = amountInputTo.getAmount();
                if (amountFrom > 0) {
                    rateNode.setRate(1.0f * amountTo / amountFrom);
                }
            }
            rateNode.updateRateInfo();
            if (amountFromChangeListener != null) {
                amountFromChangeListener.onAmountChanged(oldAmount, newAmount);
            }
        }
    };

    private final AmountInput.OnAmountChangedListener onAmountToChangedListener = new AmountInput.OnAmountChangedListener(){
        @Override
        public void onAmountChanged(long oldAmount, long newAmount) {
            long amountFrom = amountInputFrom.getAmount();
            long amountTo = amountInputTo.getAmount();
            if (amountFrom > 0) {
                rateNode.setRate(1.0f * amountTo / amountFrom);
            }
            rateNode.updateRateInfo();
            if (amountToChangeListener != null) {
                amountToChangeListener.onAmountChanged(oldAmount, newAmount);
            }
        }
    };

    public void setFromAmount(long fromAmount) {
        amountInputFrom.setAmount(fromAmount);
        calculateRate();
    }

    public void setToAmount(long toAmount) {
        amountInputTo.setAmount(toAmount);
        calculateRate();
    }

    private void updateToAmountFromRate() {
        double r = rateNode.getRate();
        long amountFrom = amountInputFrom.getAmount();
        long amountTo = (long)Math.floor(r*amountFrom);
        amountInputTo.setOnAmountChangedListener(null);
        amountInputTo.setAmount(amountTo);
        rateNode.updateRateInfo();
        amountInputTo.setOnAmountChangedListener(onAmountToChangedListener);
    }

    @Override
    public void onBeforeRateDownload() {
        amountInputFrom.setEnabled(false);
        amountInputTo.setEnabled(false);
        rateNode.disableAll();
    }

    @Override
    public String getCurrencyFrom() {
        return selectedAccountFrom != null ? selectedAccountFrom.currency.name : null;
    }

    @Override
    public String getCurrencyTo() {
        return selectedAccountTo != null ? selectedAccountTo.currency.name : null;
    }

    @Override
    public void onAfterRateDownload() {
        amountInputFrom.setEnabled(true);
        amountInputTo.setEnabled(true);
        rateNode.enableAll();
    }

    @Override
    public void onSuccessfulRateDownload() {
        updateToAmountFromRate();
    }

    @Override
    public void onRateChanged() {
        updateToAmountFromRate();
    }

    @Override
    public Activity getActivity() {
        return activity;
    }

}
